<?php

//error_reporting(E_ALL ^ E_NOTICE);

class PhpUploader
{
	/****************************************************************\
		PROPERTIES
	\****************************************************************/
	
	/// Gets or sets the maximum allowed size of the file, in KB.
	public $MaxSizeKB=-1;
	
	/// Gets or sets the allowed file extensions for uploading. 
	/// <value>A comma-separated list of allowed extensions</value>
	public $AllowedFileExtensions="";
	
	/// <summary>
	/// Gets or sets the current file name filter string, which determines the choices that appear in the file browser dialog box. 
	/// Please note that only ValidateOption.AllowedFileExtensions property determine the allowed file extensions for uploading.
	/// </summary>
	/// <remarks>
	/// <p>For each filtering option, the filter string contains a description of the filter, followed by the vertical bar (|) and the filter pattern. The strings for different filtering options are separated by the vertical bar. <br />
	/// </p>
	/// <p>The following is an example of a filter string:<br />
	/// </p>
	/// <p>Text files (*.txt)|*.txt|All files (*.*)|*.* <br />
	/// </p>
	/// <p>You can add several filter patterns to a filter by separating the file types with semicolons, for example: <br />
	/// </p>
	/// <p>Image Files(*.BMP;*.JPG;*.GIF)|*.BMP;*.JPG;*.GIF|All files (*.*)|*.* </p>
	/// </remarks>
	public $DialogFilter;
	
	/// Gets or sets the insert text.
	public $InsertText="Upload files";
	
	/// It allows the developers specify a server control to use as the insert button of ajax uploader.
	public $InsertButtonID;
	
	/// It allows the developers specify a server control to use as the cancel button of ajax uploader.
	public $CancelButtonID;
	
	/// It allows the developers specify a server control to show the progress text of ajax uploader.
	public $ProgressTextID;
	
	/// It allows the developers specify a server control to use as the progress control of ajax uploader.
	public $ProgressCtrlID;
	
  /// Gets or sets a value indicating whether the progress bar is displayed.
	public $ShowProgressBar=true;
  
	/// Gets or sets a value indicating whether the progress information is displayed when uploading.
	public $ShowProgressInfo=true;
	
	/// The global type of upload method to use.
	/// Auto|IFrame|Flash|Silverlight
	public $UploadType="Auto";
	
	/// Gets or sets a boolean indicating whether Uploader automatically starts uploading as soon as a user selects a file.
	public $ManualStartUpload=false;
	
	/// Gets or sets a boolean indicating whether Uploader should render a browse button for FireFox/Opera in Iframe Mode.
	public $ShowFrameBrowseButton=false;
	
	/// If true, the users are able to select multiple files in the file browser dialog then upload them at once. 
	public $MultipleFilesUpload=false;
	
	/// Gets or sets the maximum number of files that can be uploaded.
	public $MaxFilesLimit=-1;
	
  ///Specifies the number of files when the CancelAll button appears
	public $NumFilesShowCancelAll=2;
	
	/// set a special page for Flash - Integrated Windows Authentication 
	/// chould be also set to web.config appSettings : CuteWebUI.AjaxUploader.FlashUploadPage (CuteEditor.AjaxUploader.FlashUploadPage for CuteEditor)	
	public $FlashUploadPage;
	
	
	/// JavaScript code that be executed when the browse button be clicked.
	public $ButtonOnClickScript;
	
	/// Gets or sets the progress text template.
	/// %P% = percent,%T% = seconds remain,%F% = filename. %SEND% = uploaded size , %SIZE% = file size , %KBPS% means KB/s , %KBPS% = B/s 
	/// For example : %F%.. %P% %SEND%/%SIZE% , %KBPS% , %T% seconds left.
	public $ProgressTextTemplate;
		
	/// Gets or sets the "Cancel all Uploads" message.
	public $CancelAllMsg="Cancel all Uploads";
	
	/// Gets or sets the "Cancel upload" message.
	public $CancelUploadMsg="Cancel upload";
	
	/// Gets or sets the "upload is processing" message.
	public $UploadProcessingMsg;
	
	/// Gets or sets the "File too large" message.
	public $FileTooLargeMsg;
	
	/// Gets or sets the "maximum number of files" message.
	public $MaxFilesLimitMsg;
	
	/// Gets or sets the message for windows file browse dialog 32K file name length limitation.
	public $WindowsDialogLimitMsg;
	
	/// Gets or sets the "file type not support" message.
	public $FileTypeNotSupportMsg;
	
	/// Gets or sets the "Uploading.." message.
	public $UploadingMsg="Uploading..";
	
	/// Gets or sets the FlashWarning Image.
	public $FlashWarningImage;
	
	/// The width of the Progress Panel. Default is 360.
	public $ProgressPanelWidth=360;
	
	/// The height of the Progress bar. Default is 20.
	public $ProgressBarHeight=20;
	
	public $ProgressInfoStyle="padding-left:3px;font:normal 12px Tahoma;";
	
	/// Continuous|Blocks
	public $ProgressBarStyle="Continuous";
	
	/// Gets or sets the background image for the ProgressBar control.
	public $ProgressBarBackgroundImage;

	/// Specifies the border style for the ProgressBar control.
	public $ProgressBarBorderStyle="border:1px solid #444444;";
	
	/// Gets or sets a progress fill picture for the ProgressBar control.
	public $ProgressPicture;
  
  
  /// Specifies where AjaxUploader should put the temporary files.
	public $TempDirectory;
	
	
	/// Sets or retrieves the appearance of the file input control.
	public $InputboxCSSText="";
	
	function SetAttribute($name,$value)
	{
		$this->$name=$value;
	}
	function GetAttribute($name)
	{
		return $this->$name;
	}
	
	//__set/__get do not intercept the public fields
	function __set($propname,$propvalue)
	{
		$this->SetAttribute($propname,$propvalue);
	}
	function __get($propname)
	{
		$this->GetAttribute($propname);
	}
	
	var $_preprocessed=false;
	var $_isverify=false;
	var $_isiframemode=false;
	var $_isaddonupload=false;
	var $_fileguid=null;
	var $_filevalidating=null;
	
	public $ResourceHandler;
	public $UploadUrl;
	
	function PhpUploader()
	{
		$this->Name="AjaxUploaderFiles";
		
		$cd=dirname($this->GetWebPath(__FILE__));
		
		$this->ResourceHandler="$cd/ajaxuploaderresource.php";
		$this->UploadUrl="$cd/ajaxuploaderhandler.php";
		
		//$this->TempDirectory=dirname(dirname(__FILE__)) . "/uploadertemp";
  
		$this->TempDirectory=$this->GetSystemTempFolder();

		$this->MaxSizeKB=$this->return_bytes(ini_get('upload_max_filesize'))/1024;
	}
	
	function GetWebPath($pfile)
	{
		$ppath=$_SERVER['SCRIPT_FILENAME'];
		$vpath=$_SERVER['SCRIPT_NAME'];
		$lfile=strtolower($pfile);
		$lpath=strtolower($ppath);
		$l=min(strlen($pfile),strlen($ppath));
		for($i=0;$i<strlen($pfile);$i++)
		{
			if(substr($lfile,$i,1)!=substr($lpath,$i,1))
			{
				$vroot=substr($vpath,0,strlen($vpath)-(strlen($ppath)-$i));
				return str_replace("\\","/",$vroot . substr($pfile,$i));
			}
		}
	}
	
	function Render()
	{
		echo $this->GetString();    
	} 

	function GetString()
	{
		$this->Maintain();
				
		$resourcehandler=$this->ResourceHandler;   
		$code="";
		if(!$this->InsertButtonID)
		{
			$code.="<button id='" . $this->Name . "Button' onclick='return false;'>" . $this->InsertText . "</button>";
		}
		
		$code.="<input type='hidden' id='" . $this->Name . "' name='" . $this->Name . "' autocomplete='off' />";
		
		$code.="<script type='text/javascript' src='$resourcehandler?type=script'></script>";
		
		$code.="<img id='AjaxUploaderFiles_Loader' UniqueID='" . $this->Name . "' ";
		$code.=" Namespace='CuteWebUI' UploadModuleNotInstall='1' src='$resourcehandler?type=file&amp;file=continuous.gif'";
		
		if(!$this->InsertButtonID)
		{
			$code.=" InsertButtonID='" . $this->Name . "Button'";
		}
		
		$code.=$this->_GenerateAttribute("ResourceHandler");
		$code.=$this->_GenerateAttribute("UploadUrl");
		
		$code.=$this->_GenerateAttribute("InsertButtonID");
		$code.=$this->_GenerateAttribute("CancelButtonID");
		$code.=$this->_GenerateAttribute("ProgressTextID");
		$code.=$this->_GenerateAttribute("ProgressCtrlID");
		
		$code.=$this->_GenerateAttribute("AllowedFileExtensions");
		
		if($this->MaxSizeKB>0)$code.=$this->_GenerateAttribute("MaxSizeKB");
		
		if($this->MaxFilesLimit>0)$code.=$this->_GenerateAttribute("MaxFilesLimit");
		
		$code.=$this->_GenerateAttribute("FileTypeNotSupportMsg");
		$code.=$this->_GenerateAttribute("FileTooLargeMsg");
		$code.=$this->_GenerateAttribute("MaxFilesLimitMsg");
		$code.=$this->_GenerateAttribute("WindowsDialogLimitMsg");
		$code.=$this->_GenerateAttribute("CancelUploadMsg");
		$code.=$this->_GenerateAttribute("CancelAllMsg");
		$code.=$this->_GenerateAttribute("UploadingMsg");
		$code.=$this->_GenerateAttribute("UploadProcessingMsg");
		$code.=$this->_GenerateAttribute("FlashWarningImage");
		$code.=$this->_GenerateAttribute("FlashUploadPage");
		$code.=$this->_GenerateAttribute("DialogFilter");
		$code.=$this->_GenerateAttribute("ButtonOnClickScript");
		$code.=$this->_GenerateAttribute("UploadType");
		$code.=$this->_GenerateAttribute("ManualStartUpload","bool");
		$code.=$this->_GenerateAttribute("ShowFrameBrowseButton","bool");
		$code.=$this->_GenerateAttribute("MultipleFilesUpload","bool");
		$code.=$this->_GenerateAttribute("ShowProgressBar","bool");
		$code.=$this->_GenerateAttribute("ShowProgressInfo","bool");
		
		$code.=$this->_GenerateAttribute("NumFilesShowCancelAll");
		$code.=$this->_GenerateAttribute("ProgressTextTemplate");
		
		$code.=$this->_GenerateAttribute("ProgressPanelWidth");
		$code.=$this->_GenerateAttribute("ProgressBarHeight");
		$code.=$this->_GenerateAttribute("ProgressInfoStyle");
		$code.=$this->_GenerateAttribute("ProgressBarStyle");
		$code.=$this->_GenerateAttribute("ProgressBarBackgroundImage");
		$code.=$this->_GenerateAttribute("ProgressPicture");
		$code.=$this->_GenerateAttribute("ProgressBarBorderStyle");
		
		
		
		
		//TODO:Opera!!
		$code.=" onload='this.style.display=&quot;none&quot; ; CuteWebUI_AjaxUploader_Initialize(this.id);' onerror='this.onload()' ContextValue='TODO' />";
		return $code;
	}
	
	function _GenerateAttribute($name,$type=null)
	{
		$value=$this->$name;
		
		if(!$value)return "";
		
		switch($name)
		{
			case "AllowedFileExtensions":
				$name="Extensions";
				break;
			case "ProgressPanelWidth":
				$name="PanelWidth";
				break;
			case "ProgressBarHeight":
				$name="BarHeight";
				break;
			case "ProgressBarStyle":
				$name="BarStyle";
				break;
			case "ProgressInfoStyle":
				$name="InfoStyle";
				break;
			case "ProgressBarBackgroundImage":
				$name="BgImage";
				break;
			case "ProgressBarBorderStyle":
				$name="BorderStyle";
				break;
		}
		
		if($type=="bool")
		{
			$value=$value?"1":"0";
		}
		
		return " $name='".htmlentities($value,ENT_QUOTES)."'";
	}

	
	function _CheckProcessed()
	{
		if(!$this->_preprocessed)
			throw(new Exception("Call PreProcessRequest before access this member!"));
	}

	function IsValidationRequest()
	{
		$this->_CheckProcessed();
		return $this->_isverify;
	}
	function GetCurrentFileGuid()
	{
		$this->_CheckProcessed();
		return $this->_fileguid;
	}
	function GetValidatingFile()
	{
		if(!$this->_filevalidating)
		{
			if(!$this->IsValidationRequest())
				throw(new Exception("Current request is not validation request"));
			$this->_filevalidating = $this->_InternalGetFile($this->_fileguid,false);
			if(!$this->_filevalidating)
			{
				$dir = $this->InternalGetTempDirectory();
				throw(new Exception("Unable to find file " . $this->_fileguid . " in $dir"));
			}
		}
		return $this->_filevalidating;
	}
	function GetUploadedFile($guid)
	{
		$guid=$this->_GetGuid($guid);
		return $this->_InternalGetFile($guid,true);
	}
	function _InternalGetFile($guid,$checkValidated)
	{
		$dir = $this->InternalGetTempDirectory();
		$arr=glob("$dir/*." . $guid . ".*");
		if($arr==null||count($arr)==0)
			return null;
		if( $checkValidated && substr($this->my_base_name($arr[0]),0,10)!="persisted.")
			throw(new Exception("This file can not be validated!"));
		$mvcfile=new PhpUploadFile();
		$mvcfile->FileGuid=$guid;
		$mvcfile->FilePath=str_replace("\\","/",$arr[0]);
		$mvcfile->FileSize=filesize($arr[0]);
		$mvcfile->FileName=substr($this->my_base_name($arr[0]),47,-5);//also remove suffix ".resx"
		return $mvcfile;
	}
	
	function InternalGetTempDirectory()
	{
		$dir=$this->TempDirectory;
		$dir=str_replace("\\","/",$dir);
		if(substr($dir,strlen($dir)-1,1)=="/")
		{
			$dir=substr($dir,0,strlen($dir)-1);
		}
		return $dir;
	}
	
	function Maintain()
	{
		$dir = $this->InternalGetTempDirectory();
		$arr=glob("$dir/*.resx");
		if($arr==null)return;
		$now=time();
		foreach($arr as $filename)
		{
			$mt=filemtime($filename);
			if ( $now - $mt > 18000 ) //5 hours
			{
				unlink($filename);
			}
		}
	}
	
	function PreProcessRequest()
	{
		if($this->_preprocessed)throw(new Exception("Do not call this method twice!"));
			$this->_preprocessed=true;
		
		if(@$_SERVER['REQUEST_METHOD']!="POST")
			return false;
		
		if(@$_GET['UseUploadModule']==null)
			return false;
		
		if(@$_GET['_Namespace']!="CuteWebUI")
			return false;
		
		if(@$_GET['PageUpload']!="1")
			return false;
		
		if(@$_GET['_Addon']=="upload")
		{
			
			if(@$_GET['_GetAddonInfo']=="size")
			{
				//TODO:handle silverlight resume feature.
			}
			
			if(@$_GET['GetUploaderError']=="1")
			{
				throw(new Exception("Not Implemented"));
			}
			
			$this->_isaddonupload=true;
			try
			{
				$this->_fileguid=$this->_GetGuid($_GET['_AddonGuid']);		
				$this->_HandleAddonUpload();
			}
			catch(Exception $x)
			{
				echo("Error:" . $x->getMessage());
				exit(200);
			}
			echo("OK");
			exit(200);
		}
		else
		{
			$this->_isverify=true;
			try
			{
				if(@$_GET['_Addon']==null)
				{
					$this->_isiframemode=true;
					$this->_HandleIFrameUpload();
				}
				else
				{
					$this->_HandleAddonVerify();
				}
			}
			catch(Exception $x)
			{
				$this->_EndWithUploadError($x->getMessage());
			}
		}
		
		return true;
	}
	
	function _ValidateFile($filename,$filesize)
	{
		$maxsize=$this->MaxSizeKB;
		if($maxsize && $maxsize>0)
		{
			if($filesize > $maxsize*1024)
			{
				throw (new Exception("Error:TOOLARGE"));
			}
		}
		
		$exts=$this->AllowedFileExtensions;
		if($exts)
		{
			$extension=strtolower(pathinfo($filename,PATHINFO_EXTENSION));
			$found=false;
			$exts=split("[,;|]",$exts);
			foreach($exts as $ext)
			{
				$ext=strtolower(preg_replace("/[\\*\\.]/","",$ext));
				if($ext==$extension)
				{
					$found=true;
					break;
				}
			}
			if(!$found)
			{
				throw (new Exception("Error:INVALIDEXT"));
			}
		}
	}
	
	function _HandleAddonUpload()
	{
		if(@$_GET['_GetAddonInfo']=="size")
		{
			//TODO:handle silverlight resume feature.
		}
		
		if(@$_GET['GetUploaderError']=="1")
		{
			//PHP version do not provide more error currently..
		}

		$file="";
		foreach($_FILES as $key => $eachfile)
		{
			$file=$eachfile;
		}
		if(!$file)
		{
			throw(new Exception("Php No Request Files"));
		}

		//The PHP will convert '.' to '_'
		$addonpartial=@$_POST["CuteWebUI.AjaxUploader.Partial"];
		if(!$addonpartial)$addonpartial=@$_POST["CuteWebUI_AjaxUploader_Partial"];
		
		$existfile=$this->_InternalGetFile($this->_fileguid,false);
		
		if($addonpartial)
		{
			if($addonpartial=="Start")
			{
				if($existfile)
					throw(new Exception("Guid Error!$addonpartial"));
				$this->_MoveHttpFile($file,$this->_fileguid);
			}
			else
			{
				if(!$existfile)
					throw(new Exception("File Not Found!"));
				$this->_AppendHttpFile($file,$existfile->FilePath);
			}
		}
		else
		{
			if($existfile)
				throw(new Exception("Guid Error !"));
			$this->_MoveHttpFile($file,$this->_fileguid);
		}
		
	}
	function _HandleAddonVerify()
	{
		$this->_fileguid=$this->_GetGuid($_GET['_AddonGuid']);
		
		$mvcfile=$this->GetValidatingFile();
		
		$this->_ValidateFile($mvcfile->FileName,$mvcfile->FileSize);
	}
	function _HandleIFrameUpload()
	{
		$file="";
		foreach($_FILES as $key => $eachfile)
		{
			$file=$eachfile;
		}
		if(!$file)
		{
			throw(new Exception("No Request Files "));
		}
		
		$this->_ValidateFile($this->my_base_name($file["name"]),$file["size"]);
		
		$this->_fileguid=$this->_CreateGuid();
		
		$this->_MoveHttpFile($file,$this->_fileguid);
		
		//cache it to $this->_filevalidating
		$mvcfile=$this->GetValidatingFile();
	}

	function _MoveHttpFile($file,$guid)
	{
		$dir = $this->InternalGetTempDirectory();
		$filepath="$dir/uploading." . $guid . "." . $this->my_base_name($file["name"]) . ".resx";

		//throw(new Exception("Get file ".$file["tmp_name"]." - $filepath"));

		move_uploaded_file($file["tmp_name"],$filepath);
	}
	function _AppendHttpFile($file,$path)
	{
		$len=filesize($file["tmp_name"]);
		$src=fopen($file["tmp_name"],"r");
		$data=fread($src,$len);
		fclose($src);
		
		$dsc=fopen($path,"a");
		fwrite($dsc,$data);
		fclose($dsc);
	}

	function WriteValidationOK()
	{
		//change "uploading." to "persisted." :
		$mvcfile=$this->GetValidatingFile();
		
		$oldpath=$mvcfile->FilePath;
		$oldpath=str_replace("\\","/",$oldpath);
		$bn=$this->my_base_name($oldpath);
		if(substr($bn,0,10)=="uploading.")
		{
			$newpath=substr($oldpath,0,-strlen($bn)) . "persisted." . substr($bn,10);
			rename($oldpath,$newpath);
			$mvcfile->FilePath=$newpath;
		}

		$uploadid = $this->_GetAndCheckUploadID();
		echo("<script type='text/javascript'>");
		echo("if(window.parent.CurrentUpload)window.parent.CurrentUpload.UploadOK('" . $uploadid . "','" . $this->_fileguid . "')");
		echo("</script>");
	}
		
	function WriteValidationError($message)
	{
		//Error , delete that file?
		$mvcfile=$this->GetValidatingFile();
		
		unlink($mvcfile->FilePath);
		
		$uploadid = $this->_GetAndCheckUploadID();
		echo("<script type='text/javascript'>");
		echo("if(window.parent.CurrentUpload)window.parent.CurrentUpload.UploadError('" . $uploadid . "','" . $this->_JSEncode($message) . "')");
		echo("</script>");
	}
	
	function _EndWithUploadError($message)
	{
		$uploadid = $this->_GetAndCheckUploadID();
		echo("<script type='text/javascript'>");
		echo("if(window.parent.CurrentUpload)window.parent.CurrentUpload.UploadError('" . $uploadid . "','" . $this->_JSEncode($message) . "')");
		echo("</script>");
		exit(200);
	}
	
	function _GetAndCheckUploadID()
	{
		$id = @$_GET['_UploadID'];
		if(stripos($id,"<")!=false)throw(new Exception("Invalid _UploadID"));
		if(stripos($id,">")!=false)throw(new Exception("Invalid _UploadID"));
		return $id;
	}
	function _JSEncode($str)
	{
		$str=bin2hex($str);
		$str=preg_replace("/([\\x00-\\xFF]{2,2})/","\\x$1",$str);
		return $str;
	}
	function _GetGuid($str)
	{
		if(!preg_match("/^[0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{12}$/i",$str))
			throw(new Exception("Invalid Guid : " . $str));
		return $str;
	}
	function _CreateGuid()
	{
		//this is windows only?
		//return $this->_GetGuid(substr(com_create_guid(),1,36));
		return preg_replace_callback("/X/",create_function("",'return substr("0123456789ABCDEF",rand(0,15),1);'),"XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX");
	}
	
	
	function GetSystemTempFolder()
	{
		$str=ini_get('upload_tmp_dir');
		if($str==null||strlen($str)==0)
			$str="/tmp";
			
		if (substr($str,strlen($str)-(1))!="/")
			$str=$str."/";
		$str=$str."uploadertemp";
		if (!file_exists($str))
		{
			$result= mkdir($str,0777);
			if(!$result)
				throw(new Exception("Unable to create temp folder:" . $str));
		}
		return $str;
	}

	function return_bytes($val) { 
		$val = trim($val); 
		$last = strtolower($val[strlen($val)-1]); 
		switch($last) { 
			// The 'G' modifier is available since PHP 5.1.0 
			case 'g': 
			$val *= 1024; 
			case 'm': 
				$val *= 1024; 
			case 'k': 
				$val *= 1024; 
		} 

		return $val; 
	}	
  
	function my_base_name($path)
	{
		return basename(str_replace("\\","/",$path));
	}
}

 class PhpUploadFile
{
	public $FileGuid;
	public $FileName;
	public $FileSize;
	public $FilePath;
	function MoveTo($newpath)
	{
		rename($this->FilePath,$newpath);
	}
	function CopyTo($newpath)
	{
		copy($this->FilePath,$newpath);
	}
	function Delete()
	{
		unlink($this->FilePath);
	}
}


?>